@@ -1,9 +1,6 @@
-"""
-An interface for pint compatible to the interface of libstempo
-"""
-from __future__ import absolute_import, division, print_function
-
+"""An interface for pint compatible to the interface of libstempo."""
 import collections
+import logging
 import time
 from collections import OrderedDict
 
@@ -11,7 +8,6 @@ import astropy.constants as ac
 import astropy.coordinates.angles as ang
 import astropy.units as u
 import numpy as np
-from astropy import log
 
 import pint.models as pm
 import pint.toa as pt
@@ -19,6 +15,8 @@ from pint import fitter
 from pint.utils import has_astropy_unit
 from pint.residuals import Residuals
 
+log = logging.getLogger(__name__)
+
 __all__ = ["PINTPar", "PINTPulsar"]
 
 # Make sure we have the minuteangle and secondangle units
@@ -81,7 +79,7 @@ map_units = {
 }
 
 
-class PINTPar(object):
+class PINTPar:
     """
     Similar to the parameter class defined in libstempo, this class gives a nice
     interface to the timing model parameters
@@ -143,7 +141,7 @@ class PINTPar(object):
         self._set = value
 
 
-class PINTPulsar(object):
+class PINTPulsar:
     """
     Pulsar object class with an interface similar to tempopulsar of libstempo
     """
@@ -181,9 +179,9 @@ class PINTPulsar(object):
             Whether or not we are using the 'units' interface of libstempo
         """
         if warnings:
-            log.setLevel("INFO")
+            logging.getLogger().setLevel("INFO")
         else:
-            log.setLevel("ERROR")
+            logging.getLogger().setLevel("ERROR")
 
         self.loadparfile(parfile)
 
@@ -241,7 +239,7 @@ class PINTPulsar(object):
         if log.level < 25:
             self.t.print_summary()
 
-        self.deleted = np.zeros(self.t.ntoas, dtype=np.bool)
+        self.deleted = np.zeros(self.t.ntoas, dtype=bool)
         self._readflags()
 
     def _readflags(self):
