@@ -1,14 +1,11 @@
-# astrometry.py
-# Defines Astrometry timing model class
-from __future__ import absolute_import, division, print_function
-
+"""Astrometric models for describing pulsar sky positions."""
+import logging
 import sys
 
 import astropy.constants as const
 import astropy.coordinates as coords
 import astropy.units as u
 import numpy as np
-from astropy import log
 from astropy.time import Time
 
 from pint import ls
@@ -22,16 +19,22 @@ from pint.models.timing_model import DelayComponent, MissingParameter
 from pint.pulsar_ecliptic import OBL, PulsarEcliptic
 from pint.utils import add_dummy_distance, remove_dummy_distance
 
+log = logging.getLogger(__name__)
+
 astropy_version = sys.modules["astropy"].__version__
 mas_yr = u.mas / u.yr
 
+__all__ = ["AstrometryEquatorial", "AstrometryEcliptic", "Astrometry"]
+
 
 class Astrometry(DelayComponent):
+    """Common tools for astrometric calculations."""
+
     register = False
     category = "astrometry"
 
     def __init__(self):
-        super(Astrometry, self).__init__()
+        super().__init__()
         self.add_param(
             MJDParameter(
                 name="POSEPOCH",
@@ -113,16 +116,23 @@ class Astrometry(DelayComponent):
         available as 3-vector toa.xyz, in units of light-seconds.
         """
         tbl = toas.table
-        L_hat = self.ssb_to_psb_xyz_ICRS(epoch=tbl["tdbld"].astype(np.float64))
-        re_dot_L = np.sum(tbl["ssb_obs_pos"] * L_hat, axis=1)
-        delay = -re_dot_L.to(ls).value
-        if self.PX.value != 0.0 and np.count_nonzero(tbl["ssb_obs_pos"]) > 0:
-            L = (1.0 / self.PX.value) * u.kpc
-            # TODO: np.sum currently loses units in some cases...
-            re_sqr = (
-                np.sum(tbl["ssb_obs_pos"] ** 2, axis=1) * tbl["ssb_obs_pos"].unit ** 2
-            )
-            delay += (0.5 * (re_sqr / L) * (1.0 - re_dot_L ** 2 / re_sqr)).to(ls).value
+        delay = np.zeros(len(toas))
+        # c selects the non-barycentric TOAs that need actual calculation
+        c = np.logical_and.reduce(tbl["ssb_obs_pos"] != 0, axis=1)
+        if np.any(c):
+            L_hat = self.ssb_to_psb_xyz_ICRS(epoch=tbl["tdbld"][c].astype(np.float64))
+            re_dot_L = np.sum(tbl["ssb_obs_pos"][c] * L_hat, axis=1)
+            delay[c] = -re_dot_L.to(ls).value
+            if self.PX.value != 0.0:
+                L = (1.0 / self.PX.value) * u.kpc
+                # TODO: np.sum currently loses units in some cases...
+                re_sqr = (
+                    np.sum(tbl["ssb_obs_pos"][c] ** 2, axis=1)
+                    * tbl["ssb_obs_pos"].unit ** 2
+                )
+                delay[c] += (
+                    (0.5 * (re_sqr / L) * (1.0 - re_dot_L ** 2 / re_sqr)).to(ls).value
+                )
         return delay * u.second
 
     def get_d_delay_quantities(self, toas):
@@ -209,10 +219,18 @@ class Astrometry(DelayComponent):
 
 
 class AstrometryEquatorial(Astrometry):
+    """Astrometry in equatorial coordinates.
+
+    Parameters supported:
+
+    .. paramtable::
+        :class: pint.models.astrometry.AstrometryEquatorial
+    """
+
     register = True
 
     def __init__(self):
-        super(AstrometryEquatorial, self).__init__()
+        super().__init__()
         self.add_param(
             AngleParameter(
                 name="RAJ",
@@ -235,8 +253,8 @@ class AstrometryEquatorial(Astrometry):
             floatParameter(
                 name="PMRA",
                 units="mas/year",
-                value=0.0,
                 description="Proper motion in RA",
+                value=0.0,
             )
         )
 
@@ -244,8 +262,8 @@ class AstrometryEquatorial(Astrometry):
             floatParameter(
                 name="PMDEC",
                 units="mas/year",
-                value=0.0,
                 description="Proper motion in DEC",
+                value=0.0,
             )
         )
         self.set_special_params(["RAJ", "DECJ", "PMRA", "PMDEC"])
@@ -254,19 +272,17 @@ class AstrometryEquatorial(Astrometry):
             func = getattr(self, deriv_func_name)
             self.register_deriv_funcs(func, param)
 
-    def setup(self):
-        super(AstrometryEquatorial, self).setup()
-
     def validate(self):
-        """ Validate the input parameter.
-        """
-        super(AstrometryEquatorial, self).validate()
+        """Validate the input parameter."""
+        super().validate()
         # RA/DEC are required
         for p in ("RAJ", "DECJ"):
             if getattr(self, p).value is None:
                 raise MissingParameter("Astrometry", p)
         # Check for POSEPOCH
-        if self.POSEPOCH.quantity is None:
+        if (
+            self.PMRA.quantity != 0 or self.PMDEC.quantity != 0
+        ) and self.POSEPOCH.quantity is None:
             if self._parent.PEPOCH.quantity is None:
                 raise MissingParameter(
                     "AstrometryEquatorial",
@@ -274,7 +290,6 @@ class AstrometryEquatorial(Astrometry):
                     "POSEPOCH or PEPOCH are required if PM is set.",
                 )
             else:
-                log.warning("POSEPOCH not found; using PEPOCH unless set explicitly!")
                 self.POSEPOCH.quantity = self._parent.PEPOCH.quantity
 
     def print_par(self):
@@ -304,7 +319,7 @@ class AstrometryEquatorial(Astrometry):
         Returns
         -------
         position
-        ICRS SkyCoord object optionally with proper motion applied
+            ICRS SkyCoord object optionally with proper motion applied
 
         If epoch (MJD) is specified, proper motion is included to return
         the position at the given epoch.
@@ -349,9 +364,7 @@ class AstrometryEquatorial(Astrometry):
         return pos_icrs.transform_to(PulsarEcliptic(ecl=ecl))
 
     def coords_as_GAL(self, epoch=None):
-        """Return the pulsar's galactic coordinates as an astropy coordinate object.
-
-        """
+        """Return the pulsar's galactic coordinates as an astropy coordinate object."""
         pos_icrs = self.get_psr_coords(epoch=epoch)
         return pos_icrs.transform_to(coords.Galactic)
 
@@ -368,14 +381,14 @@ class AstrometryEquatorial(Astrometry):
         """Calculate the derivative wrt RAJ
 
         For the RAJ and DEC derivatives, use the following approximate model for
-        the pulse delay. (Inner-product between two Cartesian vectors)
+        the pulse delay. (Inner-product between two Cartesian vectors):
 
-        de = Earth declination (wrt SSB)
-        ae = Earth right ascension
-        dp = pulsar declination
-        aa = pulsar right ascension
-        r = distance from SSB to Earh
-        c = speed of light
+            - de = Earth declination (wrt SSB)
+            - ae = Earth right ascension
+            - dp = pulsar declination
+            - aa = pulsar right ascension
+            - r = distance from SSB to Earh
+            - c = speed of light
 
         delay = r*[cos(de)*cos(dp)*cos(ae-aa)+sin(de)*sin(dp)]/c
         """
@@ -471,10 +484,18 @@ class AstrometryEquatorial(Astrometry):
 
 
 class AstrometryEcliptic(Astrometry):
+    """Astrometry in ecliptic coordinates.
+
+    Parameters supported:
+
+    .. paramtable::
+        :class: pint.models.astrometry.AstrometryEcliptic
+    """
+
     register = True
 
     def __init__(self):
-        super(AstrometryEcliptic, self).__init__()
+        super().__init__()
         self.add_param(
             AngleParameter(
                 name="ELONG",
@@ -497,9 +518,9 @@ class AstrometryEcliptic(Astrometry):
             floatParameter(
                 name="PMELONG",
                 units="mas/year",
-                value=0.0,
                 description="Proper motion in ecliptic longitude",
                 aliases=["PMLAMBDA"],
+                value=0.0,
             )
         )
 
@@ -507,9 +528,9 @@ class AstrometryEcliptic(Astrometry):
             floatParameter(
                 name="PMELAT",
                 units="mas/year",
-                value=0.0,
                 description="Proper motion in ecliptic latitude",
                 aliases=["PMBETA"],
+                value=0.0,
             )
         )
 
@@ -527,19 +548,18 @@ class AstrometryEcliptic(Astrometry):
             func = getattr(self, deriv_func_name)
             self.register_deriv_funcs(func, param)
 
-    def setup(self):
-        super(AstrometryEcliptic, self).setup()
-
     def validate(self):
         """ Validate Ecliptic coordinate parameter inputs.
         """
-        super(AstrometryEcliptic, self).validate()
+        super().validate()
         # ELONG/ELAT are required
         for p in ("ELONG", "ELAT"):
             if getattr(self, p).value is None:
                 raise MissingParameter("AstrometryEcliptic", p)
         # Check for POSEPOCH
-        if self.POSEPOCH.quantity is None:
+        if (
+            self.PMELONG.value != 0 or self.PMELAT.value != 0
+        ) and self.POSEPOCH.quantity is None:
             if self._parent.PEPOCH.quantity is None:
                 raise MissingParameter(
                     "Astrometry",
@@ -547,7 +567,6 @@ class AstrometryEcliptic(Astrometry):
                     "POSEPOCH or PEPOCH are required if PM is set.",
                 )
             else:
-                log.warning("POSEPOCH not found; using PEPOCH unless set explicitly!")
                 self.POSEPOCH.quantity = self._parent.PEPOCH.quantity
 
     def barycentric_radio_freq(self, toas):
@@ -575,7 +594,6 @@ class AstrometryEcliptic(Astrometry):
 
         If epoch (MJD) is specified, proper motion is included to return
         the position at the given epoch.
-
         """
         try:
             obliquity = OBL[self.ECL.value]
@@ -585,6 +603,7 @@ class AstrometryEcliptic(Astrometry):
                 "Check your pint/datafile/ecliptic.dat file."
             )
         if epoch is None or (self.PMELONG.value == 0.0 and self.PMELAT.value == 0.0):
+            # Compute only once
             return coords.SkyCoord(
                 obliquity=obliquity,
                 lon=self.ELONG.quantity,
@@ -595,6 +614,7 @@ class AstrometryEcliptic(Astrometry):
                 frame=PulsarEcliptic,
             )
         else:
+            # Compute for each time because there is proper motion
             if isinstance(epoch, Time):
                 newepoch = epoch
             else:
@@ -611,9 +631,7 @@ class AstrometryEcliptic(Astrometry):
         return pos_ecl.transform_to(coords.ICRS)
 
     def coords_as_GAL(self, epoch=None):
-        """Return the pulsar's galactic coordinates as an astropy coordinate object.
-
-        """
+        """Return the pulsar's galactic coordinates as an astropy coordinate object."""
         pos_ecl = self.get_psr_coords(epoch=epoch)
         return pos_ecl.transform_to(coords.Galactic)
 
@@ -630,7 +648,7 @@ class AstrometryEcliptic(Astrometry):
         return pos_ecl
 
     def get_d_delay_quantities_ecliptical(self, toas):
-        """Calculate values needed for many d_delay_d_param functions """
+        """Calculate values needed for many d_delay_d_param functions."""
         # TODO: Move all these calculations in a separate class for elegance
         rd = dict()
         # From the earth_ra dec to earth_elong and elat
@@ -661,7 +679,7 @@ class AstrometryEcliptic(Astrometry):
         return result
 
     def d_delay_astrometry_d_ELONG(self, toas, param="", acc_delay=None):
-        """Calculate the derivative wrt RAJ
+        """Calculate the derivative wrt RAJ.
 
         For the RAJ and DEC derivatives, use the following approximate model for
         the pulse delay. (Inner-product between two Cartesian vectors)
