@@ -1,6 +1,4 @@
 """A simple model of a base dispersion delay and DMX dispersion."""
-from __future__ import absolute_import, division, print_function
-
 from warnings import warn
 
 import numpy as np
@@ -26,7 +24,7 @@ class Dispersion(DelayComponent):
     """A base dispersion timing model."""
 
     def __init__(self):
-        super(Dispersion, self).__init__()
+        super().__init__()
         self.dm_value_funcs = []
         self.dm_deriv_funcs = {}
 
@@ -132,13 +130,17 @@ class DispersionDM(Dispersion):
     This model uses Taylor expansion to model DM variation over time. It
     can also be used for a constant DM.
 
+    Parameters supported:
+
+    .. paramtable::
+        :class: pint.models.dispersion_model.DispersionDM
     """
 
     register = True
     category = "dispersion_constant"
 
     def __init__(self):
-        super(DispersionDM, self).__init__()
+        super().__init__()
         self.add_param(
             floatParameter(
                 name="DM",
@@ -151,7 +153,6 @@ class DispersionDM(Dispersion):
         self.add_param(
             prefixParameter(
                 name="DM1",
-                value=0.0,
                 units="pc cm^-3/yr^1",
                 description="First order time derivative of the dispersion measure",
                 unit_template=self.DM_dervative_unit,
@@ -170,7 +171,7 @@ class DispersionDM(Dispersion):
         self.delay_funcs_component += [self.constant_dispersion_delay]
 
     def setup(self):
-        super(Dispersion, self).setup()
+        super().setup()
         base_dms = list(self.get_prefix_mapping_component("DM").values())
         base_dms += ["DM"]
 
@@ -180,15 +181,18 @@ class DispersionDM(Dispersion):
 
     def validate(self):
         """Validate the DM parameters input."""
-        super(Dispersion, self).validate()
+        super().validate()
         # If DM1 is set, we need DMEPOCH
-        if self.DM1.value != 0.0:
+        if self.DM1.value is not None and self.DM1.value != 0.0:
             if self.DMEPOCH.value is None:
-                raise MissingParameter(
-                    "Dispersion",
-                    "DMEPOCH",
-                    "DMEPOCH is required if DM1 or higher are set",
-                )
+                # Copy PEPOCH (PEPOCH must be set!)
+                self.DMEPOCH.value = self._parent.PEPOCH.value
+                if self.DMEPOCH.value is None:
+                    raise MissingParameter(
+                        "Dispersion",
+                        "DMEPOCH",
+                        "DMEPOCH or PEPOCH is required if DM1 or higher are set",
+                    )
 
     def DM_dervative_unit(self, n):
         return "pc cm^-3/yr^%d" % n if n else "pc cm^-3"
@@ -198,21 +202,23 @@ class DispersionDM(Dispersion):
 
     def get_DM_terms(self):
         """Return a list of the DM term values in the model: [DM, DM1, ..., DMn]"""
-        prefix_dm = list(self.get_prefix_mapping_component("DM").values())
-        dm_terms = [self.DM.quantity]
-        dm_terms += [getattr(self, x).quantity for x in prefix_dm]
-        return dm_terms
+        return [self.DM.quantity] + self._parent.get_prefix_list("DM", start_index=1)
 
     def base_dm(self, toas):
-        tbl = toas.table
-        dm = np.zeros(len(tbl))
+        dm = np.zeros(len(toas))
         dm_terms = self.get_DM_terms()
-        if self.DMEPOCH.value is None:
-            DMEPOCH = tbl["tdbld"][0]
-        else:
+        if any(t.value != 0 for t in dm_terms[1:]):
             DMEPOCH = self.DMEPOCH.value
-        dt = (tbl["tdbld"] - DMEPOCH) * u.day
-        dt_value = (dt.to(u.yr)).value
+            if DMEPOCH is None:
+                # Should be ruled out by validate()
+                raise ValueError(
+                    f"DMEPOCH not set but some derivatives are not zero: {dm_terms}"
+                )
+            else:
+                dt = (toas["tdbld"] - DMEPOCH) * u.day
+            dt_value = dt.to_value(u.yr)
+        else:
+            dt_value = np.zeros(len(toas), dtype=np.longdouble)
         dm_terms_value = [d.value for d in dm_terms]
         dm = taylor_horner(dt_value, dm_terms_value)
         return dm * self.DM.units
@@ -221,7 +227,7 @@ class DispersionDM(Dispersion):
         """This is a wrapper function for interacting with the TimingModel class"""
         return self.dispersion_type_delay(toas)
 
-    def print_par(self,):
+    def print_par(self):
         # TODO we need to have a better design for print out the parameters in
         # an inhertance class.
         result = ""
@@ -242,14 +248,7 @@ class DispersionDM(Dispersion):
         self, toas, param_name, acc_delay=None
     ):  # NOTE we should have a better name for this.)
         """Derivatives of DM wrt the DM taylor expansion parameters."""
-        tbl = toas.table
-        try:
-            bfreq = self._parent.barycentric_radio_freq(toas)
-        except AttributeError:
-            warn("Using topocentric frequency for dedispersion!")
-            bfreq = tbl["freq"]
         par = getattr(self, param_name)
-        unit = par.units
         if param_name == "DM":
             order = 0
         else:
@@ -259,10 +258,13 @@ class DispersionDM(Dispersion):
         dm_terms = np.longdouble(np.zeros(len(dms)))
         dm_terms[order] = np.longdouble(1.0)
         if self.DMEPOCH.value is None:
-            DMEPOCH = tbl["tdbld"][0]
+            if any(t.value != 0 for t in dms[1:]):
+                # Should be ruled out by validate()
+                raise ValueError(f"DMEPOCH is not set but {param_name} is not zero")
+            DMEPOCH = 0
         else:
             DMEPOCH = self.DMEPOCH.value
-        dt = (tbl["tdbld"] - DMEPOCH) * u.day
+        dt = (toas["tdbld"] - DMEPOCH) * u.day
         dt_value = (dt.to(u.yr)).value
         d_dm_d_dm_param = taylor_horner(dt_value, dm_terms) * (
             self.DM.units / par.units
@@ -283,12 +285,17 @@ class DispersionDM(Dispersion):
         else:
             new_epoch = Time(new_epoch, scale="tdb", format="mjd", precision=9)
 
+        dmterms = [0.0 * u.Unit("")] + self.get_DM_terms()
         if self.DMEPOCH.value is None:
-            raise ValueError("DMEPOCH is not currently set.")
+            if any(d.value != 0 for d in dmterms[2:]):
+                # Should be ruled out by validate()
+                raise ValueError(
+                    f"DMEPOCH not set but some DM derivatives are not zero: {dmterms}"
+                )
+            self.DMEPOCH.value = new_epoch
 
         dmepoch_ld = self.DMEPOCH.quantity.tdb.mjd_long
         dt = (new_epoch.tdb.mjd_long - dmepoch_ld) * u.day
-        dmterms = [0.0 * u.Unit("")] + self.get_DM_terms()
 
         for n in range(len(dmterms) - 1):
             cur_deriv = self.DM if n == 0 else getattr(self, "DM{}".format(n))
@@ -303,13 +310,18 @@ class DispersionDMX(Dispersion):
 
     This model lets the user specify time ranges and fit for a different
     DM value in each time range.
+
+    Parameters supported:
+
+    .. paramtable::
+        :class: pint.models.dispersion_model.DispersionDMX
     """
 
     register = True
     category = "dispersion_dmx"
 
     def __init__(self):
-        super(DispersionDMX, self).__init__()
+        super().__init__()
         # DMX is for info output right now
         self.add_param(
             floatParameter(
@@ -319,45 +331,132 @@ class DispersionDMX(Dispersion):
                 description="Dispersion measure",
             )
         )
-        self.add_param(
-            prefixParameter(
-                name="DMX_0001",
-                units="pc cm^-3",
-                value=0.0,
-                unit_template=lambda x: "pc cm^-3",
-                description="Dispersion measure variation",
-                description_template=lambda x: "Dispersion measure",
-                paramter_type="float",
-            )
-        )
-        self.add_param(
-            prefixParameter(
-                name="DMXR1_0001",
-                units="MJD",
-                unit_template=lambda x: "MJD",
-                description="Beginning of DMX interval",
-                description_template=lambda x: "Beginning of DMX interval",
-                parameter_type="MJD",
-                time_scale="utc",
-            )
-        )
-        self.add_param(
-            prefixParameter(
-                name="DMXR2_0001",
-                units="MJD",
-                unit_template=lambda x: "MJD",
-                description="End of DMX interval",
-                description_template=lambda x: "End of DMX interval",
-                parameter_type="MJD",
-                time_scale="utc",
-            )
-        )
+
+        self.add_DMX_range(None, None, dmx=0, frozen=False, index=1)
+
         self.dm_value_funcs += [self.dmx_dm]
         self.set_special_params(["DMX_0001", "DMXR1_0001", "DMXR2_0001"])
         self.delay_funcs_component += [self.DMX_dispersion_delay]
 
+    def add_DMX_range(self, mjd_start, mjd_end, index=None, dmx=0, frozen=True):
+        """Add DMX range to a dispersion model with specified start/end MJDs and DMX.
+
+        Parameters
+        ----------
+
+        mjd_start : float
+            MJD for beginning of DMX event.
+        mjd_end : float
+            MJD for end of DMX event.
+        index : int, None
+            Integer label for DMX event. If None, will increment largest used index by 1.
+        dmx : float
+            Change in DM during DMX event.
+        frozen : bool
+            Indicates whether DMX will be fit.
+
+        Returns
+        -------
+
+        index : int
+            Index that has been assigned to new DMX event.
+
+        """
+
+        #### Setting up the DMX title convention. If index is None, want to increment the current max DMX index by 1.
+        if index is None:
+            dct = self.get_prefix_mapping_component("DMX_")
+            index = np.max(list(dct.keys())) + 1
+        i = f"{int(index):04d}"
+
+        if mjd_end is not None and mjd_start is not None:
+            if mjd_end < mjd_start:
+                raise ValueError("Starting MJD is greater than ending MJD.")
+        elif mjd_start != mjd_end:
+            raise ValueError("Only one MJD bound is set.")
+
+        if int(index) in self.get_prefix_mapping_component("DMX_"):
+            raise ValueError(
+                "Index '%s' is already in use in this model. Please choose another."
+                % index
+            )
+
+        self.add_param(
+            prefixParameter(
+                name="DMX_" + i,
+                units="pc cm^-3",
+                value=dmx,
+                description="Dispersion measure variation",
+                parameter_type="float",
+                frozen=frozen,
+            )
+        )
+        self.add_param(
+            prefixParameter(
+                name="DMXR1_" + i,
+                units="MJD",
+                description="Beginning of DMX interval",
+                parameter_type="MJD",
+                time_scale="utc",
+                value=mjd_start,
+            )
+        )
+        self.add_param(
+            prefixParameter(
+                name="DMXR2_" + i,
+                units="MJD",
+                description="End of DMX interval",
+                parameter_type="MJD",
+                time_scale="utc",
+                value=mjd_end,
+            )
+        )
+        self.setup()
+        self.validate()
+        return index
+
+    def remove_DMX_range(self, index):
+        """Removes all DMX parameters associated with a given index/list of indices.
+
+        Parameters
+        ----------
+
+        index : float, int, list, np.ndarray
+            Number or list/array of numbers corresponding to DMX indices to be removed from model.
+        """
+
+        if (
+            isinstance(index, int)
+            or isinstance(index, float)
+            or isinstance(index, np.int64)
+        ):
+            indices = [index]
+        elif not isinstance(index, list) or not isinstance(index, np.ndarray):
+            raise TypeError(
+                f"index must be a float, int, list, or array - not {type(index)}"
+            )
+        for index in indices:
+            index_rf = f"{int(index):04d}"
+            for prefix in ["DMX_", "DMXR1_", "DMXR2_"]:
+                self.remove_param(prefix + index_rf)
+        self.validate()
+
+    def get_indices(self):
+        """Returns an array of integers corresponding to DMX parameters.
+
+        Returns
+        -------
+        inds : np.ndarray
+        Array of DMX indices in model.
+        """
+        inds = []
+        for p in self.params:
+            if "DMX_" in p:
+                inds.append(int(p.split("_")[-1]))
+        return np.array(inds)
+
     def setup(self):
-        super(DispersionDMX, self).setup()
+        super().setup()
         # Get DMX mapping.
         # Register the DMX derivatives
         for prefix_par in self.get_params_of_type("prefixParameter"):
@@ -367,7 +466,7 @@ class DispersionDMX(Dispersion):
 
     def validate(self):
         """Validate the DMX parameters."""
-        super(DispersionDMX, self).validate()
+        super().validate()
         DMX_mapping = self.get_prefix_mapping_component("DMX_")
         DMXR1_mapping = self.get_prefix_mapping_component("DMXR1_")
         DMXR2_mapping = self.get_prefix_mapping_component("DMXR2_")
@@ -453,7 +552,7 @@ class DispersionDMX(Dispersion):
             dmx[v] = 1.0
         return dmx * (u.pc / u.cm ** 3) / (u.pc / u.cm ** 3)
 
-    def print_par(self,):
+    def print_par(self):
         result = ""
         DMX_mapping = self.get_prefix_mapping_component("DMX_")
         DMXR1_mapping = self.get_prefix_mapping_component("DMXR1_")
@@ -470,6 +569,11 @@ class DispersionDMX(Dispersion):
 class DispersionJump(Dispersion):
     """This class provides the contant offsets to the DM values.
 
+    Parameters supported:
+
+    .. paramtable::
+        :class: pint.models.dispersion_model.DispersionDMX
+
     Notes
     -----
     This DM jump is only for modeling the DM values, and will not apply to the
@@ -480,7 +584,7 @@ class DispersionJump(Dispersion):
     category = "dispersion_jump"
 
     def __init__(self):
-        super(DispersionJump, self).__init__()
+        super().__init__()
         self.dm_value_funcs += [self.jump_dm]
         # Dispersion jump only model the dm values.
 
@@ -494,17 +598,21 @@ class DispersionJump(Dispersion):
         )
 
     def setup(self):
-        super(DispersionJump, self).setup()
+        super().setup()
         self.dm_jumps = []
         for mask_par in self.get_params_of_type("maskParameter"):
             if mask_par.startswith("DMJUMP"):
                 self.dm_jumps.append(mask_par)
         for j in self.dm_jumps:
             self.register_dm_deriv_funcs(self.d_dm_d_dmjump, j)
-            self.register_deriv_funcs(self.d_delay_d_dmparam, j)
+            # Note we can not use the derivative function 'd_delay_d_dmparam',
+            # Since dmjump does not effect delay.
+            # The function 'd_delay_d_dmparam' applies d_dm_d_dmparam first and
+            # than applys the time delay part.
+            self.register_deriv_funcs(self.d_delay_d_dmjump, j)
 
     def validate(self):
-        super(DispersionJump, self).validate()
+        super().validate()
 
     def jump_dm(self, toas):
         """Return the DM jump for each dm section collected by dmjump parameters.
@@ -534,4 +642,5 @@ class DispersionJump(Dispersion):
 
         Since DMJUMPS does not affect delay, this would be zero.
         """
-        return np.zeros(toas.ntoas) * (u.s / self.DMJUMP.units)
+        dmjump = getattr(self, param_name)
+        return np.zeros(toas.ntoas) * (u.s / dmjump.units)
