@@ -1,22 +1,19 @@
-# model_builder.py
-# Defines the automatic timing model generator interface
-from __future__ import absolute_import, division, print_function
-
+import logging
 import os
 import tempfile
 from collections import Counter, defaultdict
 
-from astropy import log
-
+from pint.models.parameter import maskParameter
 from pint.models.timing_model import (
+    DEFAULT_ORDER,
     Component,
     TimingModel,
     ignore_prefix,
-    DEFAULT_ORDER,
 )
-from pint.models.parameter import maskParameter
-from pint.utils import PrefixError, interesting_lines, lines_of, split_prefixed_name
 from pint.toa import get_TOAs
+from pint.utils import PrefixError, interesting_lines, lines_of, split_prefixed_name
+
+log = logging.getLogger(__name__)
 
 __all__ = ["get_model"]
 
@@ -27,7 +24,7 @@ class UnknownBinaryModel(ValueError):
     """Signal that the par file requested a binary model no in PINT."""
 
 
-class ModelBuilder(object):
+class ModelBuilder:
     """A class for model construction interface.
 
     Parameters
@@ -304,7 +301,11 @@ def get_model(parfile):
     except AttributeError:
         contents = None
     if contents is None:
-        return ModelBuilder(parfile).timing_model
+        # parfile is a filename and can be handled by ModelBuilder
+        mbmodel = ModelBuilder(parfile)
+        model = mbmodel.timing_model
+        model.name = parfile
+        return model
     else:
         with tempfile.TemporaryDirectory() as td:
             fn = os.path.join(td, "temp.par")
@@ -323,6 +324,7 @@ def get_model_and_toas(
     planets=None,
     usepickle=False,
     tdb_method="default",
+    picklefilename=None,
 ):
     """Load a timing model and a related TOAs, using model commands as needed
 
@@ -345,13 +347,17 @@ def get_model_and_toas(
         Defaults to False.
     usepickle : bool
         Whether to try to use pickle-based caching of loaded clock-corrected TOAs objects.
-    tdb_method : string
-        Which method to use for the clock correction to TDB.
+    tdb_method : str
+        Which method to use for the clock correction to TDB. See
+        :func:`pint.observatory.Observatory.get_TDBs` for details.
+    picklefilename : str or None
+        Filename to use for caching loaded file. Defaults to adding ``.pickle.gz`` to the
+        filename of the timfile, if there is one and only one. If no filename is available,
+        or multiple filenames are provided, a specific filename must be provided.
 
     Returns
     -------
     A tuple with (model instance, TOAs instance)
-
     """
     mm = get_model(parfile)
     tt = get_TOAs(
@@ -363,6 +369,8 @@ def get_model_and_toas(
         include_gps=include_gps,
         planets=planets,
         usepickle=usepickle,
+        tdb_method=tdb_method,
+        picklefilename=picklefilename,
     )
     return mm, tt
 
