@@ -1,18 +1,21 @@
 """Pulsar timing noise models."""
 
-from __future__ import absolute_import, division, print_function
+import copy
+import logging
+import warnings
 
 import astropy.units as u
 import numpy as np
-from astropy import log
 
 from pint.models.parameter import floatParameter, maskParameter
 from pint.models.timing_model import Component
 
+log = logging.getLogger(__name__)
+
 
 class NoiseComponent(Component):
     def __init__(self,):
-        super(NoiseComponent, self).__init__()
+        super().__init__()
         self.covariance_matrix_funcs = []
         self.scaled_toa_sigma_funcs = []  # Need to move this to a speical place.
         self.scaled_dm_sigma_funcs = []
@@ -24,13 +27,15 @@ class NoiseComponent(Component):
         self.dm_covariance_matrix_funcs_component = []
         self.basis_funcs = []
 
-    def validate(self,):
-        super(NoiseComponent, self).validate()
-
 
 class ScaleToaError(NoiseComponent):
     """Correct reported template fitting uncertainties.
 
+    Parameters supported:
+
+    .. paramtable::
+        :class: pint.models.noise_model.ScaleToaError
+
     Note
     ----
     Ref: NanoGrav 11 yrs data
@@ -41,7 +46,7 @@ class ScaleToaError(NoiseComponent):
     category = "scale_toa_error"
 
     def __init__(self,):
-        super(ScaleToaError, self).__init__()
+        super().__init__()
         self.introduces_correlated_errors = False
         self.add_param(
             maskParameter(
@@ -78,7 +83,7 @@ class ScaleToaError(NoiseComponent):
         self.scaled_toa_sigma_funcs += [self.scale_toa_sigma]
 
     def setup(self):
-        super(ScaleToaError, self).setup()
+        super().setup()
         # Get all the EFAC parameters and EQUAD
         self.EFACs = {}
         self.EQUADs = {}
@@ -115,7 +120,7 @@ class ScaleToaError(NoiseComponent):
                     EQUAD_par.quantity = tneq_par.quantity.to(u.us)
                 else:
                     self.add_param(
-                        p.maskParameter(
+                        maskParameter(
                             name="EQUAD",
                             units="us",
                             index=tneq_par.index,
@@ -135,7 +140,7 @@ class ScaleToaError(NoiseComponent):
                 self.EQUADs[pp] = (par.key, par.key_value)
 
     def validate(self):
-        super(ScaleToaError, self).validate()
+        super().validate()
         # check duplicate
         for el in ["EFACs", "EQUADs"]:
             l = list(getattr(self, el).values())
@@ -149,10 +154,17 @@ class ScaleToaError(NoiseComponent):
             if equad.quantity is None:
                 continue
             mask = equad.select_toa_mask(toas)
-            sigma_scaled[mask] = np.hypot(sigma_scaled[mask], equad.quantity)
+            if np.any(mask):
+                sigma_scaled[mask] = np.hypot(sigma_scaled[mask], equad.quantity)
+            else:
+                warnings.warn(f"EQUAD {equad} has no TOAs")
         for efac_name in self.EFACs:
             efac = getattr(self, efac_name)
-            sigma_scaled[efac.select_toa_mask(toas)] *= efac.quantity
+            mask = efac.select_toa_mask(toas)
+            if np.any(mask):
+                sigma_scaled[mask] *= efac.quantity
+            else:
+                warnings.warn(f"EFAC {efac} has no TOAs")
         return sigma_scaled
 
     def sigma_scaled_cov_matrix(self, toas):
@@ -163,6 +175,11 @@ class ScaleToaError(NoiseComponent):
 class ScaleDmError(NoiseComponent):
     """Correction for estimated wideband DM measurement uncertainty.
 
+    Parameters supported:
+
+    .. paramtable::
+        :class: pint.models.noise_model.ScaleDmError
+
     Note
     ----
     Ref: NanoGrav 12.5 yrs wideband data
@@ -172,7 +189,7 @@ class ScaleDmError(NoiseComponent):
     category = "scale_dm_error"
 
     def __init__(self,):
-        super(ScaleDmError, self).__init__()
+        super().__init__()
         self.introduces_correlated_errors = False
         self.add_param(
             maskParameter(
@@ -198,7 +215,7 @@ class ScaleDmError(NoiseComponent):
         self._paired_DMEFAC_DMEQUAD = None
 
     def setup(self):
-        super(ScaleDmError, self).setup()
+        super().setup()
         # Get all the EFAC parameters and EQUAD
         self.DMEFACs = {}
         self.DMEQUADs = {}
@@ -214,75 +231,19 @@ class ScaleDmError(NoiseComponent):
             else:
                 continue
 
-        if len(self.DMEFACs) != len(self.DMEQUADs):
-            self._match_DMEFAC_DMEQUAD()
-        else:
-            self._paired_DMEFAC_DMEQUAD = self.pair_DMEFAC_DMEQUAD()
+        # if len(self.DMEFACs) != len(self.DMEQUADs):
+        #     self._match_DMEFAC_DMEQUAD()
+        # else:
+        #     self._paired_DMEFAC_DMEQUAD = self.pair_DMEFAC_DMEQUAD()
 
     def validate(self):
-        super(ScaleDmError, self).validate()
+        super().validate()
         # check duplicate
         for el in ["DMEFACs", "DMEQUADs"]:
             l = list(getattr(self, el).values())
             if [x for x in l if l.count(x) > 1] != []:
                 raise ValueError("'%s' have duplicated keys and key values." % el)
 
-    def _match_DMEFAC_DMEQUAD(self, add_param_to_model=False):
-        """ Match the DMEFAC and DMEQUAD parameter, if only one parameter of the
-        DMEFAC-DMEQUAD pair is given. This match is based on the parameters key
-        and key value.
-
-        Parameters
-        ----------
-        add_param_to_model: bool
-            Flags to add the parameters to the timing model instead of use the
-            default value temporarily. This is useful, if one wants to fit for
-            the match up parameters.
-        """
-        keys_and_values = {}
-        p_map = {0: (self.DMEFAC1, 1), 1: (self.DMEQUAD1, 0)}
-        keys_and_values = self.pair_DMEFAC_DMEQUAD()
-        # match params.
-        for kvs, params in keys_and_values.items():
-            if None in params:
-                p_type = params.index(None)
-                example_add_param = p_map[p_type][0]
-                pair_param = params[1 - p_type]
-                param_idx = pair_param.index
-                param_name = example_add_param.prefix + str(param_idx)
-                # search existing param but without any assigned keys
-                add_param = example_add_param.new_param(param_idx)
-                add_param.value = p_map[p_type][1]
-                add_param.key = kvs[0]
-                add_param.key_value = kvs[1]
-                params[p_type] = add_param
-                if add_param_to_model:
-                    self.add_param(add_param)
-        if add_param_to_model:
-            self.setup()
-        else:
-            self._paired_DMEFAC_DMEQUAD = keys_and_values
-
-    # pairing up EFAC and EQUAD
-    def pair_DMEFAC_DMEQUAD(self):
-        """ Pair the DMEFAC and DMEQUAD.
-        """
-        keys_and_values = {}
-        # Check the dm efac first
-        for dmefac, efac_key in self.DMEFACs.items():
-            if efac_key[0] is not None:
-                keys_and_values[efac_key] = [getattr(self, dmefac), None]
-        # Check the dm equad then
-        for dmequad, equad_key in self.DMEQUADs.items():
-            if equad_key[0] is not None:
-                # Add matches.
-                if equad_key in keys_and_values.keys():
-                    keys_and_values[equad_key][1] = getattr(self, dmequad)
-                else:
-                    keys_and_values[equad_key] = [None, getattr(self, dmequard)]
-
-        return keys_and_values
-
     def scale_dm_sigma(self, toas):
         """
         Scale the DM uncertainty.
@@ -293,17 +254,18 @@ class ScaleDmError(NoiseComponent):
             Input DM error object. We assume DM error is stored in the TOA
             objects.
         """
-        sigma_old = toas.get_dm_errors()
-        sigma_scaled = np.zeros_like(sigma_old)
-        if self._paired_DMEFAC_DMEQUAD is None:
-            self.setup()
-        for pir in self._paired_DMEFAC_DMEQUAD.values():
-            efac = pir[0]
-            equad = pir[1]
-            mask = efac.select_toa_mask(toas)
-            sigma_scaled[mask] = efac.quantity * np.sqrt(
-                sigma_old[mask] ** 2 + (equad.quantity) ** 2
-            )
+        sigma_scaled = copy.deepcopy(toas.get_dm_errors())
+        # Apply DMEQUAD first
+        for dmequad_name in self.DMEQUADs:
+            dmequad = getattr(self, dmequad_name)
+            if dmequad.quantity is None:
+                continue
+            mask = dmequad.select_toa_mask(toas)
+            sigma_scaled[mask] = np.hypot(sigma_scaled[mask], dmequad.quantity)
+        # Then apply the DMEFAC
+        for dmefac_name in self.DMEFACs:
+            dmefac = getattr(self, dmefac_name)
+            sigma_scaled[dmefac.select_toa_mask(toas)] *= dmefac.quantity
         return sigma_scaled
 
     def dm_sigma_scaled_cov_matrix(self, toas):
@@ -319,6 +281,11 @@ class EcorrNoise(NoiseComponent):
     and forth within the average profile, and this effect is the same
     for all frequencies. Thus these TOAs have correlated errors.
 
+    Parameters supported:
+
+    .. paramtable::
+        :class: pint.models.noise_model.EcorrNoise
+
     Note
     ----
     Ref: NanoGrav 11 yrs data
@@ -329,7 +296,7 @@ class EcorrNoise(NoiseComponent):
     category = "ecorr_noise"
 
     def __init__(self,):
-        super(EcorrNoise, self).__init__()
+        super().__init__()
         self.introduces_correlated_errors = True
         self.add_param(
             maskParameter(
@@ -346,7 +313,7 @@ class EcorrNoise(NoiseComponent):
         self.basis_funcs += [self.ecorr_basis_weight_pair]
 
     def setup(self):
-        super(EcorrNoise, self).setup()
+        super().setup()
         # Get all the EFAC parameters and EQUAD
         self.ECORRs = {}
         for mask_par in self.get_params_of_type("maskParameter"):
@@ -357,7 +324,7 @@ class EcorrNoise(NoiseComponent):
                 continue
 
     def validate(self):
-        super(EcorrNoise, self).validate()
+        super().validate()
 
         # check duplicate
         for el in ["ECORRs"]:
@@ -384,7 +351,11 @@ class EcorrNoise(NoiseComponent):
         umats = []
         for ec in ecorrs:
             mask = ec.select_toa_mask(toas)
-            umats.append(create_quantization_matrix(t[mask]))
+            if np.any(mask):
+                umats.append(create_quantization_matrix(t[mask]))
+            else:
+                warnings.warn(f"ECORR {ec} has no TOAs")
+                umats.append(np.zeros((0, 0)))
         nc = sum(u.shape[1] for u in umats)
         umat = np.zeros((len(t), nc))
         weight = np.zeros(nc)
@@ -413,6 +384,11 @@ class PLRedNoise(NoiseComponent):
     is noise dominated by the lowest frequency. This results in errors
     that are correlated between TOAs over fairly long time spans.
 
+    Parameters supported:
+
+    .. paramtable::
+        :class: pint.models.noise_model.PLRedNoise
+
     Note
     ----
     Ref: NanoGrav 11 yrs data
@@ -423,7 +399,7 @@ class PLRedNoise(NoiseComponent):
     category = "pl_red_noise"
 
     def __init__(self,):
-        super(PLRedNoise, self).__init__()
+        super().__init__()
         self.introduces_correlated_errors = True
         self.add_param(
             floatParameter(
@@ -470,12 +446,6 @@ class PLRedNoise(NoiseComponent):
         self.covariance_matrix_funcs += [self.pl_rn_cov_matrix]
         self.basis_funcs += [self.pl_rn_basis_weight_pair]
 
-    def setup(self):
-        super(PLRedNoise, self).setup()
-
-    def validate(self):
-        super(PLRedNoise, self).validate()
-
     def get_pl_vals(self):
         nf = int(self.TNRedC.value) if self.TNRedC.value is not None else 30
         if self.TNRedAmp.value is not None and self.TNRedGam.value is not None:
